/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.mock;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.ServletException;
import javax.servlet.http.Cookie;

import org.apache.commons.io.IOUtils;
import org.loom.action.Action;
import org.loom.log.Log;
import org.loom.persistence.file.PersistentFile;
import org.loom.resolution.ForwardResolution;
import org.loom.resolution.RedirectResolution;
import org.loom.resolution.Resolution;
import org.loom.servlet.CacheControl;
import org.loom.servlet.LoomServletRequest;
import org.loom.servlet.LoomServletResponse;
import org.loom.url.UrlBuilder;

import com.google.common.collect.ArrayListMultimap;
import com.google.common.collect.ListMultimap;
import com.google.common.collect.Lists;

/**
 * A mock Resolution class
 * @author icoloma
 *
 */
public class MockResolution implements Resolution, RedirectResolution, ForwardResolution {

	/** forward/redirect */
	private String type;
	
	/** the url to be forwarded */
	private UrlBuilder url;
	
	/** the file to be sent */
	private PersistentFile file;
	
	/** true to write the sent file to disk in a temporary file, for manual checking */
	private boolean writeFile;
	
	private CacheControl cacheControl;
	
	private List<Cookie> cookies = Lists.newArrayList();
	
	private Map<String, Object> attributes = new HashMap<String, Object>();
	
	private static Log log = Log.getLog();
	
	private static MockResolution lastResolution;
	
	private ListMultimap<String, Object> headers = ArrayListMultimap.create();
	
	private String charset;
	
	public static MockResolution getLastResolution() {
		return lastResolution;
	}
	
	public MockResolution(String type, String url) {
		this.type = type;
		this.url = new UrlBuilder();
		this.url.setRawUrl(url);
		lastResolution = this;
	}
	
	public MockResolution(String type, UrlBuilder builder) {
		this.type = type;
		this.url = builder;
		lastResolution = this;
	}

	public MockResolution(PersistentFile file, boolean writeFile) {
		this.file = file;
		this.writeFile = writeFile;
		lastResolution = this;
	}

	public MockResolution(Class<? extends Action> actionClass, String event) {
		lastResolution = this;
	}

	public void resolve(LoomServletRequest request, LoomServletResponse response) throws IOException, ServletException {
		if (url != null) {
			log.info("Resolution ", type, ": ", url.getURL());
		} else if (file != null) {
			log.info("Resolution file sent: [", file.getFilename(), ", content-type=", file.getContentType(), 
					", file-size=", file.getFileSize(), "bytes]");
			if (writeFile) {
				writeFile();
			}
		} else {
			throw new RuntimeException("All the resolution fields are empty. Specify a url or a file!");
		}
			
	}
	
	public String getForwardedUrl() {
		return url.getRawUrl();
	}
	
	private void writeFile() throws IOException, FileNotFoundException {
		File f = File.createTempFile("resolution", file.getFilename());
		FileOutputStream out = null;
		try {
			out = new FileOutputStream(f);
			IOUtils.copy(file.getContents().getContentsAsStream(), out);
		} finally {
			log.info("File contents are written to ", f.getAbsoluteFile());
			IOUtils.closeQuietly(out);
		}
	}

	/**
	 * Adds a parameter to the redirection url
	 */
	public MockResolution addParameter(String name, Object value) {
		url.add(name, value);
		return this;
	}
	
	/**
	 * Adds a parameter to the redirection url
	 */
	public MockResolution addPropertyParameter(String property) {
		return addParameter(property, property + " value");
	}

	public MockResolution setHash(String anchor) {
		url.setHash(anchor);
		return this;
	}

	public String getType() {
		return type;
	}

	public UrlBuilder getUrl() {
		return url;
	}

	public PersistentFile getFile() {
		return file;
	}

	public MockResolution setCacheControl(CacheControl cacheControl) {
		this.cacheControl = cacheControl;
		return this;
	}

	public CacheControl getCacheControl() {
		return cacheControl;
	}

	public List<String> removeParameter(String name) {
		return url.remove(name);
	}

	public MockResolution replaceParameter(String name, Object value) {
		url.replace(name, value);
		return this;
	}

	public MockResolution setScheme(String scheme) {
		url.setScheme(scheme);
		return this;
	}

	public MockResolution setHostname(String hostname) {
		url.setHostname(hostname);
		return this;
	}

	public MockResolution setPort(Integer port) {
		url.setPort(port);
		return this;
	}

	public MockResolution setIncludeRequestParameters(
			boolean addCurrentRequestParameters) {
		url.setIncludeRequestParameters(addCurrentRequestParameters);
		return this;
	}

	public ForwardResolution setAttribute(String name, Object value) {
		attributes.put(name, value);
		return this;
	}
	
	public MockResolution permanent() {
		return this;
	}

	@Override
	public MockResolution add(String name, Object value) {
		attributes.put(name, value);
		return this;
	}

	@Override
	public MockResolution addCookie(Cookie cookie) {
		cookies.add(cookie);
		return this;
	}

	@Override
	public MockResolution addCookie(String name, String value) {
		Cookie cookie = new Cookie(name, value);
		return addCookie(cookie);
	}

	@Override
	public MockResolution addCookie(String name, String value, int ttl) {
		Cookie cookie = new Cookie(name, value);
		cookie.setMaxAge(ttl);
		return addCookie(cookie);
	}

	@Override
	public MockResolution addHeader(String name, Object value) {
		headers.put(name, value);
		return this;
	}

	public MockResolution addProperty(String arg0) {
		throw new UnsupportedOperationException();
	}

	@Override
	public List<String> remove(String name) {
		return url.remove(name);
	}

	@Override
	public MockResolution replace(String name, Object value) {
		url.replace(name, value);
		return this;
	}

	@Override
	public MockResolution setCharset(String charset) {
		this.charset = charset;
		return this;
	}

	@Override
	public MockResolution setContentType(String arg0) {
		return this;
	}

	@Override
	public MockResolution setHeader(String name, Object value) {
		headers.removeAll(name);
		headers.put(name, value);
		return this;
	}

	@Override
	public MockResolution setHeaderIfNotPresent(String name, Object value) {
		if (!headers.containsKey(name))
			headers.put(name, value);
		return this;
	}

	@Override
	public MockResolution set(String name, Object value) {
		attributes.put(name, value);
		return this;
	}

	public List<Cookie> getCookies() {
		return cookies;
	}

	public String getCharset() {
		return charset;
	}

}
